def save_fasta_file(sequences: list[str], file_path: str):
    with open(file_path, "w") as fasta_file:
        for i, sequence in enumerate(sequences):
            fasta_file.write(">Sequence_" + str(i) + "\n")
            fasta_file.write(sequence + "\n")



import Levenshtein

from plcg.sequence_scoring.sequence_to_number_conversions import num_list_to_aa_seq


def get_sequence_diff(sequence_one: list[int], sequence_two: str) -> float:
    sequence_one = num_list_to_aa_seq(sequence_one)
    sequence_diff = Levenshtein.distance(sequence_one, sequence_two)
    return sequence_diff


def get_min_sequence_diff(s1: list[list[int]], s2: str) -> float:
    s1 = [num_list_to_aa_seq(x) for x in s1]
    sequence_diff = [Levenshtein.distance(x, s2) for x in s1]
    return min(sequence_diff)



from calendar import c
import os
import shutil
import subprocess
import xml.etree.ElementTree as ET


def run_sp_server(
    input_filepath: str, output_filepath: str, spserver_path_parent_dir: str
):
    subprocess.run(
        [
            "python",
            spserver_path_parent_dir + "spserver/SPServerPPI.py",
            "-i",
            input_filepath,
            "-r",
            "A",
            "-l",
            "B",
            "-s",
            "pdb_together",
            "-o",
            output_filepath,
            "-j",
            "spserverscores",
            "-p",
            "CB",
            "-c",
        ]
    )


def read_spserver_xml(filepath: str):
    tree = ET.parse(filepath)
    root = tree.getroot()

    global_energies = root.find(".//global_energies")

    final_lst = []
    values = {}
    for child in global_energies:
        values[child.tag] = child.text
        if child.text != "NA":
            final_lst.append(float(child.text))
    return final_lst


def cleanup_spserver_files(spserver_path_parent_dir: str):
    if os.path.exists(spserver_path_parent_dir + "working_spserver_pdb/workingpdb.pdb"):
        os.remove(spserver_path_parent_dir + "working_spserver_pdb/workingpdb.pdb")
    if os.path.exists(spserver_path_parent_dir + "working_spserver_output/"):
        shutil.rmtree(spserver_path_parent_dir + "working_spserver_output/")


def calculate_sp_server_scores(pdb, spserver_path_parent_dir: str):
    pdb_filepath = "working_spserver_pdb/workingpdb.pdb"
    cleanup_spserver_files(spserver_path_parent_dir)
    with open(spserver_path_parent_dir + pdb_filepath, "wb") as output_file:
        output_file.write(bytes(pdb[0]))

    run_sp_server(
        spserver_path_parent_dir + pdb_filepath,
        spserver_path_parent_dir + "workingspserver/",
        spserver_path_parent_dir,
    )
    scores = read_spserver_xml(
        spserver_path_parent_dir + "workingspserver/spserverscores.xml"
    )
    cleanup_spserver_files(spserver_path_parent_dir)

    return scores



import csv
import os
import shutil
import subprocess
import json

from plcg.structure_prediction.fasta.fasta import save_fasta_file


def run_colabfold(input_filepath: str, output_dir: str):
    subprocess.call(
        [
            "colabfold_batch",
            input_filepath,
            output_dir,
            "--stop-at-score",
            "88.0",
            "--num-recycle",
            "4",
        ]
    )


def run_colabfold_and_get_filepath(seq: str, working_dir: str):
    if os.path.exists(working_dir):
        shutil.rmtree(working_dir)
    os.mkdir(working_dir)
    with open(working_dir + "working.csv", "w", encoding="utf-8") as f:
        write = csv.writer(f)
        write.writerows([["id", "sequence"], ["sequence", seq]])
    files = os.listdir(working_dir)
    flag = 0
    while flag == 0:
        pdb_filepath = [result for result in files if "unrelaxed_rank_001" in result]
        if len(pdb_filepath) > 0:
            pdb_filepath = pdb_filepath[0]
            flag = 1
        else:
            pdb_filepath = [result for result in files if "unrelaxed_" in result]
            if len(pdb_filepath) > 0:
                pdb_filepath = pdb_filepath[0]
                flag = 1
            else:
                run_colabfold(working_dir + "working.csv", working_dir)
                files = os.listdir(working_dir)

    json_filepath = [result for result in files if "scores_rank_001" in result]
    ptm_value = None
    if len(json_filepath) > 0:
        json_filepath = json_filepath[0]
        with open(working_dir + json_filepath, "r", encoding="utf-8") as f:
            data = json.load(f)
            ptm_value = data.get("ptm")

    return pdb_filepath, ptm_value


def colabfold_batch(fasta_filepath: str, output_dir: str):
    subprocess.call(
        [
            "colabfold_batch",
            fasta_filepath,
            output_dir,
            "--stop-at-score",
            "88.0",
            "--num-recycle",
            "4",
        ]
    )


def run_colabfold_batch_and_return_filepaths(sequences: list[str], working_dir: str):
    if os.path.exists(working_dir):
        shutil.rmtree(working_dir)
    os.mkdir(working_dir)
    fasta_file_path = "output.fasta"
    save_fasta_file(sequences, working_dir + fasta_file_path)
    colabfold_batch(working_dir + fasta_file_path, working_dir)
    sequences_to_pdb_filepaths = {}
    sequences_to_json_filepaths = {}
    files = os.listdir(working_dir)
    for i, sequence in enumerate(sequences):
        results = [result for result in files if "Sequence_" + str(i) in result]
        result = [result for result in results if "unrelaxed_rank_001" in result]
        json_filepath = [result for result in results if "scores_rank_001" in result]
        if len(result) > 0:
            result = result[0]
            json_filepath = json_filepath[0]
            sequences_to_pdb_filepaths[i] = working_dir + result
            sequences_to_json_filepaths[i] = working_dir + json_filepath
    return sequences_to_pdb_filepaths, sequences_to_json_filepaths



import subprocess
import os
import pandas as pd


def calculate_rosetta_scores(
    sequence: str, pdb, working_dir: str, python_rosetta_interpreter_path: str
):
    pdb_filepath = os.path.join(working_dir, "workingpdb.pdb")
    if os.path.exists(pdb_filepath):
        os.remove(pdb_filepath)
    with open(pdb_filepath, "wb") as output_file:
        output_file.write(bytes(pdb[0]))
    return run_rosetta(sequence, pdb_filepath, python_rosetta_interpreter_path)


def run_rosetta(sequence: str, pdb_path: str, python_rosetta_interpreter_path: str):
    script_dir = os.path.dirname(__file__)
    get_rosetta_scores_script_path = os.path.join(
        script_dir, "calculate__rosetta_scores.py"
    )
    cached_extra_seqs_dir = "cached-extra-seqs"
    os.makedirs(cached_extra_seqs_dir, exist_ok=True)

    subprocess.run(
        [
            python_rosetta_interpreter_path,
            get_rosetta_scores_script_path,
            "seq_then_path",
            sequence,
            pdb_path,
        ],
        stdout=subprocess.DEVNULL,
        stderr=subprocess.PIPE,
        text=True,
    )
    csv_path = os.path.join(cached_extra_seqs_dir, sequence + ".csv")
    read = pd.read_csv(csv_path)
    os.remove(csv_path)
    return read.to_numpy()[0].tolist()



# pylint: skip-file
from pyrosetta import *
from pyrosetta.rosetta.protocols.docking import setup_foldtree
from pyrosetta.rosetta.protocols.docking import DockMCMProtocol
from pyrosetta.rosetta.core.scoring import ScoreType
import numpy as np
import json
from pyrosetta.rosetta.core.scoring.hbonds import HBondSet


def get_energy(scorefxn, pose):
    feature_vector = {}
    for score_type in scorefxn.get_nonzero_weighted_scoretypes():
        feature_vector[str(score_type)] = scorefxn.score_by_scoretype(pose, score_type)
    return feature_vector


def get_per_residue_energy(pose):
    per_residue_energy = {}
    for i in range(1, pose.total_residue() + 1):
        res_energy = pose.energies().residue_total_energy(i)
        per_residue_energy[i] = res_energy

    energies = list(per_residue_energy.values())

    total_energy = sum(energies)
    average_energy = sum(energies) / len(energies)
    max_energy = max(energies)
    min_energy = min(energies)
    std_dev_energy = np.std(energies)
    return {
        "total_energy": total_energy,
        "average_energy": average_energy,
        "max_energy": max_energy,
        "min_energy": min_energy,
        "std_dev_energy": std_dev_energy,
    }


def get_interface_analyzer(pose):
    from pyrosetta.rosetta.protocols.analysis import InterfaceAnalyzerMover

    interface_analyzer = InterfaceAnalyzerMover()
    interface_analyzer.apply(pose)

    interface_data = {
        "interface_delta_G": interface_analyzer.get_interface_dG(),
        "dSASA": sum(interface_analyzer.get_all_per_residue_data().dSASA),
        "num_interface_residues": interface_analyzer.get_num_interface_residues(),
        "complexed_sasa": interface_analyzer.get_complexed_sasa(),
        "total_hbond_energy": interface_analyzer.get_total_Hbond_E(),
        "separated_energy": sum(
            interface_analyzer.get_all_per_residue_data().separated_energy
        ),
        "separated_sasa": sum(
            interface_analyzer.get_all_per_residue_data().separated_sasa
        ),
    }
    return interface_data


def get_per_res_hydrophobic_sasa(pose):
    rsd_sasa = pyrosetta.rosetta.utility.vector1_double()
    rsd_hydrophobic_sasa = pyrosetta.rosetta.utility.vector1_double()
    return {
        "per_res_hydrophobic_sasa": rosetta.core.scoring.calc_per_res_hydrophobic_sasa(
            pose, rsd_sasa, rsd_hydrophobic_sasa, 1.4
        )
    }


def get_rsd_hydrophobic_sasa(pose):
    rsd_sasa = pyrosetta.rosetta.utility.vector1_double()
    rsd_hydrophobic_sasa = pyrosetta.rosetta.utility.vector1_double()
    rosetta.core.scoring.calc_per_res_hydrophobic_sasa(
        pose, rsd_sasa, rsd_hydrophobic_sasa, 1.4
    )

    for i in range(1, pose.size() + 1):
        total_sasa = rosetta.core.scoring.normalizing_area_total_hydrophobic_atoms_only(
            pose.residue(i).name1()
        )
        rsd_hydrophobic_sasa[i] = total_sasa - rsd_hydrophobic_sasa[i]
    return {"rsd_hydrophobic_sasa": sum(rsd_hydrophobic_sasa)}


def get_buhs_for_each_res(pose):
    bupc = rosetta.protocols.simple_pose_metric_calculators.BuriedUnsatisfiedPolarsCalculator(
        "default", "default"
    )

    sfxn = rosetta.core.scoring.get_score_function()
    sfxn(pose)

    buhs_for_each_res = json.loads(bupc.get("residue_bur_unsat_polars", pose))
    return {"buhs_for_each_res": sum(buhs_for_each_res)}


def get_hbond_info(pose):
    hbond_set = HBondSet(pose, False)
    hbonds = [(hbond.don_res(), hbond.acc_res()) for hbond in hbond_set.hbonds()]
    hbond_count_per_residue = [0] * pose.total_residue()
    for hbond in hbonds:
        donor_res, acc_res = hbond
        hbond_count_per_residue[donor_res - 1] += 1
        hbond_count_per_residue[acc_res - 1] += 1
    hbond_count_per_residue

    count = len([x for x in hbond_count_per_residue if x > 0])
    avg = sum(hbond_count_per_residue) / len(hbond_count_per_residue)

    return {"hbond_count": count, "hbond_avg": avg}



from plcg.constants.values import AMINO_ACIDS


def num_list_to_aa_seq(amino_acid_number_list: list[int]) -> str:
    """Convert a sequence of numbers corresponding to amino acids into a sequence
    of amino acids (in form of a string)

    - input: A sequence of numbers where each number is from 0 to 19
    """
    return "".join([AMINO_ACIDS[int(num)] for num in amino_acid_number_list])


def aa_seq_to_num_list(seq: str) -> list[int]:
    """Convert an amino acid sequence into a sequence of numbers from 0 to 19

    - seq: A string consisting of an amino acid sequence where each amino acid is
    a capitalized character
    """
    return [AMINO_ACIDS.index(aa) for aa in seq]


def num_to_aa(num: int) -> str:
    return AMINO_ACIDS[int(num)]


def aa_to_num(aa: str) -> int:
    return AMINO_ACIDS.index(aa)



import numpy as np


def moving_average(x: np.ndarray, w: int) -> np.ndarray:
    return np.convolve(x, np.ones(w), "valid") / w


def sigmoid(x):
    return 1 / (1 + np.exp(-x))


def softmax(x: np.ndarray) -> np.ndarray:
    return np.exp(x) / np.nansum(np.exp(x))



import os


def get_plddt(pdb, working_dir: str):
    plddt_sum = 0
    length = 0
    pdb_filepath = "working_plddt.pdb"
    if os.path.exists(working_dir + pdb_filepath):
        os.remove(working_dir + pdb_filepath)
    with open(working_dir + pdb_filepath, "wb") as output_file:
        output_file.write(bytes(pdb[0]))

    with open(working_dir + pdb_filepath, "r") as file:
        for line in file:
            split = line.split()
            if len(split) > 10:
                plddt_sum += float(split[10])
                length += 1
    os.remove(working_dir + pdb_filepath)
    return plddt_sum / length



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="plcg",
    version="1.0.3",
    author="Spencer Perkins",
    author_email="spencer.perkins44sp@gmail.com",
    description="Computational biology and machine learning utilities for the Pardee Lab Computation Group",
    project_urls={
        "Code": "https://github.com/Pardee-Lab-Computation-Group/plcg",
        "Bug Tracker": "https://github.com/Pardee-Lab-Computation-Group/plcg/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=find_packages(
        where="src",
        include=["plcg*"],
    ),
    package_dir={"": "src"},
    python_requires=">=3.10",
    long_description_content_type="text/markdown",
    long_description="A variety of python utilities for biology",
    install_requires=["pandas==2.2.0", "numpy==1.26.3", "python-Levenshtein==0.23.0"],
)



# pylint: skip-file
import os
import sys
import pandas as pd
from pyrosetta import *
from score_calculations import *


def rosetta_scores(sequence):
    pose = pose_from_pdb()
    scorefxn = get_fa_scorefxn()
    dict_list = [
        get_energy(scorefxn, pose),
        get_per_residue_energy(pose),
        get_interface_analyzer(pose),
        get_per_res_hydrophobic_sasa(pose),
        get_rsd_hydrophobic_sasa(pose),
        get_buhs_for_each_res(pose),
        get_hbond_info(pose),
    ]
    return dict_list


def get_all_rosetta_scores():
    init()

    if sys.argv[1] == "seq":
        seq = sys.argv[2]

        file_name = seq + ".csv"
        file_path = os.path.join("./rosetta-scores-two/rosetta-scorespdb/", file_name)

        if os.path.exists(file_path):
            df = pd.read_csv(file_path)
            print(df.iloc[0, 1:])
            return df.iloc[0, 1:]
        else:
            print("sequence not found, try seq_then_path")
    elif sys.argv[1] == "seq_then_path":
        seq = sys.argv[2]
        path = sys.argv[3]

        file_name = seq + ".csv"
        file_path = os.path.join("./rosetta-scores-two/rosetta-scorespdb/", file_name)
        file_path2 = os.path.join("./cached-extra-seqs/", file_name)

        if os.path.exists(file_path):
            df = pd.read_csv(file_path)
            print(df.iloc[0, 1:], file=sys.stderr)
            return df.iloc[0, 1:]
        elif os.path.exists(file_path2):
            df = pd.read_csv(file_path2)
            print(df.iloc[0, 1:], file=sys.stderr)
            return df.iloc[0, 1:]
        else:
            pose = pose_from_pdb(path)
            scorefxn = get_fa_scorefxn()

            dict_list = [
                get_energy(scorefxn, pose),
                get_per_residue_energy(pose),
                get_interface_analyzer(pose),
                get_per_res_hydrophobic_sasa(pose),
                get_rsd_hydrophobic_sasa(pose),
                get_buhs_for_each_res(pose),
                get_hbond_info(pose),
            ]

            df_list = [pd.DataFrame([d]) for d in dict_list]

            final_df = pd.concat(df_list, axis=1)

            os.makedirs("./cached-extra-seqs/", exist_ok=True)
            if not os.path.exists(os.path.join("./cached-extra-seqs/", seq + ".csv")):
                final_df.to_csv("./cached-extra-seqs/" + seq + ".csv")
            print(final_df.iloc[0, :], file=sys.stderr)
            return final_df.iloc[0, :]
    elif sys.argv[1] == "path":
        seq = sys.argv[2]
        pdb_path = sys.argv[3]
        pose = pose_from_pdb(pdb_path)
        scorefxn = get_fa_scorefxn()

        dict_list = [
            get_energy(scorefxn, pose),
            get_per_residue_energy(pose),
            get_interface_analyzer(pose),
            get_per_res_hydrophobic_sasa(pose),
            get_rsd_hydrophobic_sasa(pose),
            get_buhs_for_each_res(pose),
            get_hbond_info(pose),
        ]

        df_list = [pd.DataFrame([d]) for d in dict_list]

        final_df = pd.concat(df_list, axis=1)
        if not os.path.exists(os.path.join("./cached-extra-seqs/", seq + ".csv")):
            final_df.to_csv("./cached-extra-seqs/" + seq + ".csv")
        print(final_df.iloc[0, :], file=sys.stderr)
        return final_df.iloc[0, :]


if __name__ == "__main__":
    get_all_rosetta_scores()
